@@ -40,6 +40,8 @@ import subprocess
 import sys
 import tempfile
 from pathlib import Path
+import uuid
+import configparser
 
 import jsonschema
 import yaml
@@ -53,13 +55,22 @@ from ansible_bender.schema import PLAYBOOK_SCHEMA
 from ansible_bender.utils import run_cmd, ap_command_exists, random_str, graceful_get, \
     is_ansibles_python_2
 
+from ansible.inventory.manager import InventoryManager
+from ansible.vars.manager import VariableManager
+from ansible.parsing.dataloader import DataLoader
+from ansible_bender.conf import Build, ImageMetadata
+
 logger = logging.getLogger(__name__)
+# callback_whitelist got renamed to callbacks_enabled in ansible
+# drop callback_whitelist once 2.15 is released
+# https://docs.ansible.com/ansible/latest/reference_appendices/config.html#callbacks-enabled
 A_CFG_TEMPLATE = """\
 [defaults]
 # when user is changed, ansible might not be able to write to /.ansible
 remote_tmp = /tmp
 callback_plugins={0}
 callback_whitelist=snapshoter\n
+callbacks_enabled=snapshoter\n
 """
 
 
@@ -173,6 +184,30 @@ class AnsibleRunner:
         # hence, let's add the site ab is installed in to sys.path
         return os.path.dirname(os.path.dirname(ansible_bender.__file__))
 
+    def _correct_host_entries(self, playbook_path, tmpDir):
+        """ Correct the host entries in the playbook and all imported playbooks """
+        tmp_pb_path = os.path.join(tmpDir, "ab_" + str(uuid.uuid4()) + ".yaml")
+
+        with open(playbook_path, "r") as fd_r:
+            pb_dict = yaml.safe_load(fd_r)
+            
+        for idx, doc in enumerate(pb_dict):
+            imported_playbook = doc.get("import_playbook")
+            if imported_playbook:
+                import_base_path = os.path.dirname(playbook_path)
+                imported_playbook_path = os.path.join(import_base_path, imported_playbook)
+                logger.debug("Encountered import_playbook, correcting hosts entries in imported file: %s", imported_playbook_path)
+                doc["import_playbook"] = self._correct_host_entries(imported_playbook_path, tmpDir)
+            else:
+                host = doc["hosts"]
+                logger.debug("play[%s], host = %s", idx, host)
+                doc["hosts"] = self.builder.ansible_host
+
+        with open(tmp_pb_path, "w") as fd:
+            yaml.safe_dump(pb_dict, fd)
+
+        return tmp_pb_path
+    
     def build(self, db_path):
         """
         run the playbook against the container
@@ -204,15 +239,7 @@ class AnsibleRunner:
             with open(a_cfg_path, "w") as fd:
                 self._create_ansible_cfg(fd)
 
-            tmp_pb_path = os.path.join(tmp, "p.yaml")
-            with open(self.pb, "r") as fd_r:
-                pb_dict = yaml.safe_load(fd_r)
-            for idx, doc in enumerate(pb_dict):
-                host = doc["hosts"]
-                logger.debug("play[%s], host = %s", idx, host)
-                doc["hosts"] = self.builder.ansible_host
-            with open(tmp_pb_path, "w") as fd:
-                yaml.safe_dump(pb_dict, fd)
+            tmp_pb_path = self._correct_host_entries(self.pb, tmp)
             playbook_base = os.path.basename(self.pb).split(".", 1)[0]
             timestamp = datetime.datetime.now().strftime(TIMESTAMP_FORMAT)
             symlink_name = f".{playbook_base}-{timestamp}-{random_str()}.yaml"
@@ -244,38 +271,12 @@ class PbVarsParser:
         self.metadata = ImageMetadata()
         self.build.metadata = self.metadata
 
-    def _check_selinux_iz_gud(self):
-        """
-        This is a workaround for a weird behavior of ansible: if selinux is
-        in the enforcing mode and python3-libselinux is not installed, ansible freezes
-
-        https://bugzilla.redhat.com/show_bug.cgi?id=1696706
-        :return:
-        """
-        try:
-            enforcing_status = Path("/sys/fs/selinux/enforce").read_text()
-        except FileNotFoundError:
-            logger.debug("this system is not using selinux, /sys/fs/selinux/enforce is not present")
-            return
-        logger.debug(f"selinux enforce status = {enforcing_status}")
-        # it can be enforcing or not, selinux python module needs to be present
-        try:
-            importlib.import_module("selinux")
-        except ModuleNotFoundError:
-            raise RuntimeError(
-                "\nThis system is using selinux(8) and selinux python module is not installed. "
-                "There is a known issue in ansible that it freezes in this setup:\n"
-                "  https://bugzilla.redhat.com/show_bug.cgi?id=1696706\n"
-                "Please install libselinux python bindings (on Fedora the package name is python3-libselinux)."
-            )
-
     def expand_pb_vars(self):
         """
         populate vars from a playbook, defined in vars section
 
         :return: dict with the content of ansible_bender var
         """
-        self._check_selinux_iz_gud()
         with open(self.playbook_path) as fd:
             plays = yaml.safe_load(fd)
 
@@ -367,7 +368,7 @@ class PbVarsParser:
             # validation to error out unknown keys in /vars/ansible_bender
             jsonschema.validate(bender_data, PLAYBOOK_SCHEMA)
         except jsonschema.ValidationError as validation_error:
-            if validation_error.validator is "type":
+            if validation_error.validator == "type":
                 # error is due to invalid value datatype
                 path = "/" + "/".join(validation_error.path)
                 expected_types = validation_error.validator_value
@@ -394,3 +395,221 @@ class PbVarsParser:
         self.process_pb_vars(bender_data)
 
         return self.build, self.metadata
+
+
+class AnsibleVarsParser:
+    """
+    AnsibleVarParser is used to parse the ansible variables from the playbook and inventory
+    This class attempts to use the ansible API to parse the variables
+    If the inventory is not specified, the class attempts to find the inventory file
+    If the inventory is not found, the class will fall back to PbVarsParser to parse the variables
+    """
+
+    # Prefix for ansible_bender variables
+    _variable_prefix = "ansible_bender"
+
+    _inventory_path = None
+
+    _playbook_path = None
+
+    _loader: DataLoader = None
+
+    _inventory: InventoryManager = None
+
+    _variable_manager: VariableManager = None
+
+    _hosts_in_playbook: list = None
+
+    _hosts: dict = {}
+
+    build: Build = None
+
+    metadata: ImageMetadata = None
+
+    def __init__(self, playbook_path, inventory_path = None) -> None:
+        self._base_dir = os.path.dirname(playbook_path)
+        self._playbook_path = playbook_path
+        self._inventory_path = inventory_path if inventory_path else self._find_inventory_path()
+        self.build = Build()
+        self.metadata = ImageMetadata()
+        self.build.metadata = self.metadata
+
+    def __getitem__(self, key):
+        return self._hosts[key]
+
+    def _find_ansible_cfg(self) -> str:
+        """
+        Find the ansible.cfg file
+        Precendence: https://docs.ansible.com/ansible/latest/reference_appendices/config.html
+        """
+        # Check environment variable
+        if "ANSIBLE_CONFIG" in os.environ:
+            return os.environ["ANSIBLE_CONFIG"]
+
+        # Check in self._base_dir (playbook directory)
+        if os.path.isfile(os.path.join(self._base_dir, "ansible.cfg")):
+            return os.path.join(self._base_dir, "ansible.cfg")
+
+        # Check in home directory
+        if os.path.isfile(os.path.join(os.path.expanduser("~"), ".ansible.cfg")):
+            return os.path.join(os.path.expanduser("~"), ".ansible.cfg")
+
+        # Check in /etc/ansible/ansible.cfg
+        if os.path.isfile("/etc/ansible/ansible.cfg"):
+            return "/etc/ansible/ansible.cfg"
+
+        # No ansible.cfg file found
+        return None
+
+    def _find_inventory_path(self) -> str:
+        """
+        Get the inventory path from either
+        1. ansible.cfg
+        2. the default ansible inventory location (/etc/ansible/hosts)
+        """
+        ansible_cfg_path = self._find_ansible_cfg()
+
+        if ansible_cfg_path:
+            return self._get_inventory_from_ansible_cfg(ansible_cfg_path)
+        else:
+            if os.path.isfile("/etc/ansible/hosts"):
+                return"/etc/ansible/hosts"
+
+        return None
+
+    def _get_inventory_from_ansible_cfg(self, ansible_cfg_path) -> str:
+        """
+        Get the inventory path from the ansible.cfg file
+        """
+        config = configparser.ConfigParser()
+        config.read(ansible_cfg_path)
+        inv_path = config.get("defaults", "inventory", fallback=None)
+
+        # Possible to have an ansible.cfg file without an inventory path
+        if inv_path is None:
+            return None
+
+        # Inventory path can be absolute or relative in ansible.cfg
+        if os.path.isabs(inv_path):
+            return inv_path
+        else:
+            return os.path.join(os.path.dirname(ansible_cfg_path), inv_path)
+
+    def _get_host_names(self, hosts) -> list:
+        """
+        Parse the hosts from the playbook
+        """
+        parsed_host_information = []
+
+        for host in hosts:
+            parsed_host_information.append(host.get("hosts"))
+
+        return parsed_host_information
+
+    def _extract_variables(self, ansible_variables, variables):
+        """
+        Convinience function to get the variables from the variables dictionary
+        Gets the ansible_bender variables and the prepended variables
+        """
+
+        if variables is None:
+            return ansible_variables
+
+        for key, value in variables.items():
+            if str(key).startswith(self._variable_prefix):
+                    # Assume that ansible API has already 
+                    # pulled variables with correct precedence
+                new_key = key.replace(self._variable_prefix, "")
+
+                    # key can be prepended with ansible_bender_ or ansible_bender
+                    # if it is prepended with ansible_bender_, remove the _
+                if new_key.startswith("_"):
+                    new_key = new_key[1:]
+
+                    # If the new_key is blank
+                    # then the "ansible_bender" variable was used
+                    # Add the variables to the ansible_variables
+                if new_key == "":
+                    ansible_variables.update(value)
+                else:
+                    ansible_variables[new_key] = value
+
+        return ansible_variables
+
+    def _get_vars_from_playbook(self, host_name) -> dict:
+        """
+        Parse the playbook variables related to the ansible_bender information
+        """
+        playbook_variables = {}
+
+        try:
+            for host in self._hosts_in_playbook:
+                if host.get("hosts") == host_name:
+                    playbook_variables = self._extract_variables(playbook_variables, host.get("vars"))
+        except Exception as e:
+            print("Error getting playbook variables for host: " + host_name + " - " + str(e))
+            pass
+
+        return playbook_variables
+
+    def _get_vars_for_host(self, host_name) -> dict:
+        """
+        Parse the host variables from the inventory information
+        """
+        ansible_variables = {}
+        try:
+            ansible_host = self._inventory.get_host(host_name)
+            all_vars_for_host = self._variable_manager.get_vars(host=ansible_host, 
+                                                            include_hostvars=True, 
+                                                            include_delegate_to=True)
+
+            # find prepended variables
+            self._extract_variables(ansible_variables, all_vars_for_host)                
+        except Exception as e:
+            print("Error getting host and group vars for host: " + host_name + " - " + str(e))
+            pass
+
+        # Append the playbook variables to the ansible variables
+        ansible_variables.update(self._get_vars_from_playbook(host_name))
+
+        return ansible_variables
+
+    
+    def get_build_and_metadata(self):
+        """
+        Find all hosts in the playbook and get their variables
+        return the build and imagemetadata information
+        """
+        if self._inventory_path:
+            # Ansible API objects used to pull variables
+            relative_playbook_file_path = os.path.basename(self._playbook_path)
+            self._loader = DataLoader()
+            self._loader.set_basedir(self._base_dir)
+            self._inventory = InventoryManager(loader=self._loader, sources=self._inventory_path)
+            self._variable_manager = VariableManager(loader=self._loader, inventory=self._inventory)
+            self._hosts_in_playbook = self._loader.load_from_file(relative_playbook_file_path)
+
+            host_names = self._get_host_names(self._hosts_in_playbook)
+            for host_name in host_names:
+                ansible_vars = self._get_vars_for_host(host_name)
+                self._hosts[host_name] = ansible_vars
+
+            # We only support one host in the playbook
+            host = self._hosts[list(self._hosts)[0]] if len(self._hosts) > 0 else None
+
+            if not host:
+                raise Exception("No hosts found in the playbook")
+
+            self.metadata.update_from_configuration(host.get("target_image", {}))
+            self.build.update_from_configuration(host)
+        else:
+            # At this point
+            # 1. No inventory was provided
+            # 2. No ansible.cfg file was found with inventory information
+            # 3. No default inventory file was found
+            # So forget about using the ansible API to get variables
+            # Just manually parse the playbook
+            pb_vars_p = PbVarsParser(self._playbook_path)
+            self.build, self.metadata = pb_vars_p.get_build_and_metadata()
+
+        return self.build, self.metadata
